<?php

declare(strict_types=1);

namespace Drupal\reviewer_test_kit\Plugin\reviewer\Task\Entity\Display;

use Drupal\Core\Entity\Display\EntityDisplayInterface;
use Drupal\reviewer\Attribute\Task;

/**
 * Internal task for checking the field group settings on entity display forms.
 *
 * Field types are the field group formatter plugin names.
 *
 * @internal
 *   This class is internal and the methods within may change at any time. Use
 *   \Drupal\reviewer_test_kit\Plugin\reviewer\Task\Entity\Display\Form\FormFieldGroupSettingsTaskBase
 *   or
 *   \Drupal\reviewer_test_kit\Plugin\reviewer\Task\Entity\Display\View\ViewFieldGroupSettingsTaskBase
 *   to check if fields are disabled on specific entity displays instead.
 *
 * @see \Drupal\reviewer_test_kit\Plugin\reviewer\Task\Entity\Display\Form\FormFieldGroupSettingsTaskBase
 * @see \Drupal\reviewer_test_kit\Plugin\reviewer\Task\Entity\Display\View\ViewFieldGroupSettingsTaskBase
 */
#[Task(module_dependencies: ['field_group'])]
abstract class FieldGroupSettingsTaskBase extends FieldSettingsTaskBase {

  /**
   * Get all field names matching the field type and optional handler.
   *
   * @return string[]
   */
  protected function fieldNamesFromTypes(EntityDisplayInterface $display): array {
    // We can short-circuit here if types are empty or all types are matched.
    if (!$this->fieldTypes) {
      return [];
    }

    if ($this->fieldTypes === ['*']) {
      return array_keys($this->fieldsCallback($display));
    }

    // Get all field group format type matches.
    return array_keys(array_filter(
      $this->fieldsCallback($display),
      fn(array $group) => \in_array($group['format_type'] ?? '', $this->fieldTypes, TRUE),
    ));
  }

  /**
   * {@inheritdoc}
   */
  protected function fieldsCallback(EntityDisplayInterface $display): array {
    return $display->getThirdPartySettings('field_group');
  }

  /**
   * {@inheritdoc}
   */
  protected function fixSaveCallback(
    EntityDisplayInterface $display,
    string $field,
    array $settings,
  ): void {
    $display->setThirdPartySetting('field_group', $field, $settings);
  }

}
