<?php

declare(strict_types=1);

namespace Drupal\reviewer_test_kit\Plugin\reviewer\Task\Entity\Display\Form;

use Drupal\reviewer_test_kit\Plugin\reviewer\Task\Entity\Display\FieldsDisabledTaskBase;

/**
 * Base task for checking if fields are disabled on entity form display forms.
 *
 * Tasks extending this class only need to define one or both of the $fieldTypes
 * or $fieldNames properties; the task takes care of checking and fixing
 * non-hidden fields automatically.
 *
 * If either the $fieldTypes or $fieldNames properties are empty, a
 * \LogicException will be thrown.
 *
 * Example of a fixable task which checks that fields are disabled:
 *
 * @code
 * #[Task('unused_fields_disabled')]
 * final class UnusedFieldsDisabled extends FormFieldsDisabledTaskBase {
 *
 *   protected array $fieldTypes = [
 *     'langcode',
 *     'promote',
 *     'status',
 *     'sticky',
 *   ];
 *
 *   protected array $fieldNames = [
 *     'field_disable_me',
 *   ];
 *
 * }
 * @endcode
 *
 * @see \Drupal\reviewer_test_kit\Plugin\reviewer\Task\Entity\Display\DisplayTaskBase::$displayType
 * @see \Drupal\reviewer_test_kit\Plugin\reviewer\Task\Entity\Display\DisplayFieldsTaskBase::$fieldTypes
 * @see \Drupal\reviewer_test_kit\Plugin\reviewer\Task\Entity\Display\DisplayFieldsTaskBase::$fieldNames
 */
abstract class FormFieldsDisabledTaskBase extends FieldsDisabledTaskBase {

  /**
   * {@inheritdoc}
   */
  protected string $displayType = 'form';

}
