<?php

declare(strict_types=1);

namespace Drupal\reviewer_test_kit\Plugin\reviewer\Task;

/**
 * A trait to make parsing nested configuration values a little easier.
 */
trait NestedConfigTrait {

  /**
   * Return all expected values to be iterated over.
   *
   * @param array<string, array|scalar> $expected_values
   *   The settings to parse.
   * @param string[] &$parents
   *   All parent key values needed to find this setting value in the array.
   * @param int $depth
   *   The current depth of traversal.
   *
   * @return \Generator<scalar>
   */
  protected function expectedValues(
    array $expected_values,
    array &$parents,
    int $depth = 0,
  ): \Generator {
    foreach ($expected_values as $key => $value) {
      $parents = array_slice($parents, 0, $depth);
      $parents[$depth] = $key;

      if (\is_array($value)) {
        yield from $this->expectedValues($value, $parents, $depth + 1);
      }
      else {
        yield $value;
      }
    }
  }

  /**
   * Check if a value is negated.
   *
   * @phpstan-assert-if-true string $value
   */
  protected function negated(mixed $value): bool {
    return \is_string($value) && str_starts_with($value, '!');
  }

  /**
   * Strip the negation from a negated value.
   */
  protected function stripNegation(mixed $value): mixed {
    return $this->negated($value) ? ltrim($value, '!') : $value;
  }

}
