<?php

namespace Drupal\reviewer\Plugin\reviewer\Review;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\reviewer\Plugin\reviewer\BuildableInterface;

/**
 * Defines a review plugin which reviews configuration.
 */
interface ReviewPluginInterface extends BuildableInterface, PluginInspectionInterface {

  /**
   * Get the human-readable label.
   */
  public function getLabel(): string;

  /**
   * Get the checklist plugin classes this review should run.
   *
   * @return class-string<\Drupal\reviewer\Reviewer\Checklist\ChecklistInterface>[]
   *
   * @throws \Drupal\reviewer\Exception\NoChecklistsException
   *   Thrown when no checklists are defined in the review.
   */
  public function getChecklists(): array;

  /**
   * Get the specific config entity ID to review.
   */
  public function getConfigEntityId(): string|null;

  /**
   * Get the bundles of the entity type to review.
   *
   * @return string[]
   */
  public function getBundles(): array;

  /**
   * Get failed task IDs to ignore.
   *
   * @return array{id: string, reason: string}[]
   */
  public function getIgnored(): array;

}
