<?php

declare(strict_types=1);

namespace Drupal\reviewer\Reviewer\Result;

use Drupal\reviewer\Reviewer\Status\Status;

/**
 * A result.
 *
 * Use \Drupal\reviewer\Reviewer\Result\ResultFactory to create results.
 * Creating results directly using 'new Result()' will not set the status of
 * ignored items.
 */
final readonly class Result implements IndividualResultInterface {

  // phpcs:ignore Drupal.Commenting.FunctionComment.Missing
  public function __construct(
    private string $id,
    private Status $status,
    private string $message,
    private ?bool $fixable = NULL,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getId(): string {
    return $this->id;
  }

  /**
   * {@inheritdoc}
   */
  public function getStatus(): Status {
    return $this->status;
  }

  /**
   * {@inheritdoc}
   */
  public function getMessage(): string {
    return $this->message;
  }

  /**
   * {@inheritdoc}
   */
  public function isFixable(): bool {
    return $this->fixable ?? FALSE;
  }

}
