<?php

declare(strict_types=1);

namespace Drupal\reviewer\Reviewer\Status;

use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Represents the status of a result.
 *
 * Cases backed with larger integers are considered more severe than cases
 * backed with smaller integers.
 *
 * @see \Drupal\reviewer\Reviewer\Status\StatusFactoryInterface::createMostSevere
 */
enum Status: int {

  case NotRun = 0;
  case Pass = 1;
  case Fixed = 2;
  case IgnoredFailure = 3;
  case IgnoredError = 4;
  case Fail = 5;
  case Error = 6;

  /**
   * A human-readable label for the status.
   */
  public function label(): TranslatableMarkup {
    return match ($this) {
      Status::NotRun => t('Not Run'),
      Status::Pass => t('Passed'),
      Status::Fixed => t('Fixed'),
      Status::IgnoredFailure => t('Ignored Failure'),
      Status::IgnoredError => t('Ignored Error'),
      Status::Fail => t('Failed'),
      Status::Error => t('Errored'),
    };
  }

}
