<?php

namespace Drupal\reviewer\Reviewer\Status;

/**
 * Defines a factory which creates statuses.
 */
interface StatusFactoryInterface {

  /**
   * Create an ignored status from a status.
   */
  public function createIgnored(Status $status): Status;

  /**
   * Create an unignored status from a status.
   */
  public function createUnignored(Status $status): Status;

  /**
   * Create a status from a boolean.
   */
  public function createFromBool(bool $bool): Status;

  /**
   * Return the most severe status from a set of results.
   *
   * The most severe status is the case backed by the highest integer.
   *
   * @param array<\Drupal\reviewer\Reviewer\Result\ResultInterface|\Drupal\reviewer\Reviewer\Result\CollectionResultInterface> $results
   */
  public function createMostSevere(array $results): Status;

}
