<?php

declare(strict_types=1);

namespace Drupal\reviewer_test\Plugin\reviewer\Task;

use Drupal\reviewer\Reviewer\Task\TaskBase;

/**
 * Base task for testing.
 */
abstract class TestBaseTask extends TaskBase {

  /**
   * Get all displays for the configuration entity the task is checking.
   *
   * @return array<string, \Drupal\Core\Entity\Display\EntityViewDisplayInterface>
   */
  protected function getDisplays(): array {
    $entity = $this->getConfigEntity();

    if (
      !$entity
      || !$entity->getEntityType()->getBundleOf()
      || !$entity->getOriginalId()
    ) {
      return [];
    }

    $enabled_displays = array_keys($this->entityDisplayRepository->getViewModeOptionsByBundle(
      $entity->getEntityType()->getBundleOf(),
      (string) $entity->getOriginalId(),
    ));

    $displays = [];
    foreach ($enabled_displays as $mode) {
      $displays[$mode] = $this->entityDisplayRepository->getViewDisplay(
        $entity->getEntityType()->getBundleOf(),
        (string) $entity->getOriginalId(),
        $mode,
      );
    }
    return $displays;
  }

}
